Exception: XRPL::FaucetError
- Inherits:
-
StandardError
- Object
- StandardError
- XRPL::FaucetError
- Defined in:
- lib/xrpl/faucet.rb
Overview
Raised when a faucet request fails or funding does not complete in time.
Raised when a faucet request fails or funding does not complete in time.